unit KnobsDesigner;

interface

uses

  System.Types, System.Rtti, Vcl.Controls, System.Classes;


type

  TNotifyReason = (
    nrStructureChanged,
    nrPublicChanged   ,
    nrNameChanged     ,
    nrValueChanged    ,
    nrTextChanged
  );


  TNotifyReasons   = set of TNotifyReason;
  IKnobsDesignable = interface;


  IKnobsDesigner = interface ['{D2636C37-B407-4B57-84BE-19681E18570F}']
    procedure   NotifyAbout( const aTarget: IKnobsDesignable; const aReasons: TNotifyReasons; const aValue: TValue);
  end;


  IKnobsDesignable = interface ['{FE1E91EC-207D-4371-98E0-38BF7E6D0162}']
    function    GetDesigning    : Boolean;
    procedure   SetDesigning    ( aValue: Boolean);
    function    GetIsPublishable: Boolean;
    function    GetIsPublic     : Boolean;
    procedure   SetIsPublic     ( aValue: Boolean);
    function    GetDesigner     : IKnobsDesigner;
    procedure   SetDesigner     ( const aValue: IKnobsDesigner);
    function    GetExtent       : TRect;
    procedure   SetExtent       ( const aValue: TRect);
    function    GetCtrlName     : string;
    procedure   SetCtrlName     ( const aValue: string);
    function    GetModuleName   : string;

    property    Designing     : Boolean        read GetDesigning     write SetDesigning;
    property    IsPublishable : Boolean        read GetIsPublishable;
    property    IsPublic      : Boolean        read GetIsPublic      write SetIsPublic;
    property    Designer      : IKnobsDesigner read GetDesigner      write SetDesigner;
    property    Extent        : TRect          read GetExtent        write SetExtent;
    property    CtrlName      : string         read GetCtrlName      write SetCtrlName;
    property    ModuleName    : string         read GetModuleName;
  end;


  function  MakeCtrlName( const aPrefix, aSuffix: string): string;
  function  GetCtrlNamePrefix( const aCtrlName: string): string;
  function  GetCtrlNameSuffix( const aCtrlName: string): string;
  function  NotifyReasonToStr ( aReason : TNotifyReason ): string;
  function  NotifyReasonsToStr( aReasons: TNotifyReasons): string;
  function  Clone( const aSrc: TControl; const anOwner: TComponent; const aParent: TWinControl): TControl;


implementation

const

  CNAME_SEPARATOR = '_';


  function  MakeCtrlName( const aPrefix, aSuffix: string): string;
  begin
    Result := aPrefix + CNAME_SEPARATOR + aSuffix;
  end;


  function  GetCtrlNamePrefix( const aCtrlName: string): string;
  var
    p : Integer;
  begin
    p := Pos( CNAME_SEPARATOR, aCtrlName);

    if p > 0
    then Result := Copy( aCtrlName, Low( aCtrlName), Low( aCtrlName) + p - 2)
    else Result := aCtrlName;
  end;


  function  GetCtrlNameSuffix( const aCtrlName: string): string;
  var
    p : Integer;
  begin
    p := Pos( CNAME_SEPARATOR, aCtrlName);

    if p > 0
    then Result := Copy( aCtrlName, Low( aCtrlName) + p + Length( CNAME_SEPARATOR) - 1, Length( aCtrlName))
    else Result := '';
  end;


  function  NotifyReasonToStr( aReason : TNotifyReason): string;
  begin
    Result := TRttiEnumerationType.GetName( aReason);
  end;


  function  NotifyReasonsToStr( aReasons: TNotifyReasons): string;
  var
    r : TNotifyReason;
  begin
    Result := '';

    for r in aReasons
    do begin
      if Result = ''
      then Result :=                 NotifyReasonToStr( r)
      else Result := Result + ', ' + NotifyReasonToStr( r)
    end;

    if Result = ''
    then Result := '[]'
    else Result := '[ ' + Result + ']';
  end;


  function  Clone( const aSrc: TControl; const anOwner: TComponent; const aParent: TWinControl): TControl;
  var
    aStream : TMemoryStream;
    OldName : string;
  begin
    Assert( Assigned( aSrc) and Assigned( aParent) and ( aSrc.Owner <> anOwner));
    Result  := TControlClass( aSrc.ClassType).Create( anOwner);
    aStream := TMemoryStream.Create;

    try
      OldName   := aSrc.Name;
      aSrc.Name := '';

      try
        aStream.WriteComponent( aSrc);
        aStream.Position := 0;
        aStream.ReadComponent( Result);

      finally
        aStream.DisposeOf;
      end;
    finally
      aSrc.Name := OldName;
    end;

    aParent.InsertControl( Result);
  end;



end.

